#Required libraries
library(moments)
library(FD)

####################################################################
#Functions for a single step of community assembly
####################################################################

#Function to randomly remove one species.
#com.dist should be a class dist object of
#a dissimilarity matrix of trait(s) in a species pool
#that includes distances from an optimum trait value(s) named "opt" as the last row
remove_randomly=function(com.dist){
	#Transform com.dist into a matrix
	com.mat <- as.matrix(com.dist)
	com.ori.mat <- com.mat
	#Remove the row named "opt"
	non.opt <- !(grepl("opt",colnames(com.mat)))
	com.mat <- com.mat[non.opt,non.opt]
	#Remove one species (Choose n-1 species)
	exist <- sample(colnames(com.mat),ncol(com.mat)-1)
	com.mat <- com.mat[exist,exist]

	#The last row "opt" should be restored.
	opt.row <- com.ori.mat["opt",colnames(com.mat)]
	opt.col <- c(com.ori.mat[rownames(com.mat),"opt"],0)
	com.mat <- rbind(com.mat,opt.row)
	com.mat <- cbind(com.mat,opt.col)
	rownames(com.mat)[nrow(com.mat)] <- "opt"
	colnames(com.mat)[ncol(com.mat)] <- "opt"
	#Restore com.mat into class dist object
	res <- as.dist(com.mat)
	return(res)
	}

#A function to remove one species based on environment
remove_via_filt=function(com.dist){
	#Transform com.dist into a matrix
	com.mat <- as.matrix(com.dist)
	#Judge whether a row is "opt" or not
	non.opt <- !(grepl("opt",colnames(com.mat)))
	#List distances from an optimum trait value
	dist.opt <- com.mat["opt",non.opt]
	#Assign id of the most distant species to ext1
	ext1 <- which(dist.opt==max(dist.opt))
	#Sample one species randomly if tie exists
	ext1 <- ifelse(length(ext1)>=2,sample(ext1,1),ext1)
	#Assign a name of the selected species to ext
	ext <- names(dist.opt)[ext1]
	#Remove the species from the species pool
	exist <- colnames(com.mat)!=ext
	res <- as.dist(com.mat[exist,exist])
	return(res)
	}

#A function to remove one species based on similarity.
remove_via_comp=function(com.dist){
	#Transform com.dist into a matrix
	com.mat <- as.matrix(com.dist)
	#Remove the row named "opt"
	com.ori.mat <- com.mat
	non.opt <- !(grepl("opt",colnames(com.mat)))
	com.mat <- com.mat[non.opt,non.opt]
	#Restore com.mat into class dist object
	com.dist <- as.dist(com.mat)
	#List all pairs with a minimum neighbor distance
	min.dist.pair <- which(com.mat==min(com.dist), arr.ind=TRUE)
	#Diagonal elements should be excluded from the list
	min.dist.pair <- subset(min.dist.pair,(min.dist.pair[,1]-min.dist.pair[,2])!=0)
	#Choose one species to exclude
	ext <- sample(rownames(min.dist.pair),1)
	#Choose n-1 existing species
	exist <- colnames(com.mat)!=ext
	com.mat <- com.mat[exist,exist]

	#The last row "opt" should be restored.
	opt.row <- com.ori.mat["opt",colnames(com.mat)]
	opt.col <- c(com.ori.mat[rownames(com.mat),"opt"],0)
	com.mat <- rbind(com.mat,opt.row)
	com.mat <- cbind(com.mat,opt.col)
	rownames(com.mat)[nrow(com.mat)] <- "opt"
	colnames(com.mat)[ncol(com.mat)] <- "opt"
	#Restore com.mat into class dist object
	res <- as.dist(com.mat)
	return(res)
	}

#A function to randomly add one species.
#com.ori.dist is a class dist object of
#a dissimilarity matrix of trait(s) in a species pool
#that includes distances from an ideal trait value(s) named "opt" as the last row
#com.dist is the subset generated in the last iteration
add_randomly=function(com.ori.dist,com.dist=NULL){
	#Transform com.dist into a matrix
	com.ori.mat <- as.matrix(com.ori.dist)
	#Remove the row named "opt"
	com.ori.opt <- com.ori.mat
	non.opt <- !(grepl("opt",colnames(com.ori.mat)))
	com.ori.mat <- com.ori.mat[non.opt,non.opt]
	#if it is the first iteration
	if(is.null(com.dist)) {
		#Choose the first species randomly
		seed <- sample(colnames(com.ori.mat),1)
		com.mat <- as.matrix(com.ori.mat[seed,seed])
		colnames(com.mat) <- seed
		rownames(com.mat) <- seed
	}else{
	#if it is not the first iteration
	#Remove the row named "opt"
		com.mat <- as.matrix(com.dist)
		non.opt <- !(grepl("opt",colnames(com.mat)))
		com.mat <- com.mat[non.opt,non.opt]
	}
	#Assign names of candidate species to cand
	cand <- colnames(com.ori.mat)[!colnames(com.ori.mat) %in% colnames(com.mat)] 

	#Choose one species randomly
	add <- ifelse(length(cand)>=2,sample(cand,1),cand)

	#Assign names of species in a local community to exist
	exist <- c(add,colnames(com.mat))
	#Generate a new local community
	com.mat <- com.ori.mat[exist,exist]

	#The last row "opt" should be restored.
	opt.row <- com.ori.opt["opt",colnames(com.mat)]
	opt.col <- c(com.ori.opt[rownames(com.mat),"opt"],0)
	com.mat <- rbind(com.mat,opt.row)
	com.mat <- cbind(com.mat,opt.col)
	rownames(com.mat)[nrow(com.mat)] <- "opt"
	colnames(com.mat)[ncol(com.mat)] <- "opt"
	#Restore com.mat into class dist object
	res <- as.dist(com.mat)
	return(res)
	}

#A function to add one species based on environment
#com.ori.dist is a class dist object of
#a dissimilarity matrix of trait(s) in a species pool
#that includes distances from an ideal trait value(s) named "opt" as the last row
#com.dist is the subset generated in the last iteration
add_via_filt=function(com.ori.dist,com.dist=NULL){
	com.ori.mat <- as.matrix(com.ori.dist)
	com.ori.opt <- com.ori.mat
	#if it is the first iteration
	if(is.null(com.dist)) {
		#Judge whether a row is "opt" or not
		non.opt <- !(grepl("opt",colnames(com.ori.mat)))
		#List distances from an optimum trait value
		dist.opt <- com.ori.mat["opt",non.opt]
		#Assign id of the most distant species to seed1
		seed1 <- which(dist.opt==min(dist.opt))
		#Sample one species randomly if tie exists
		seed1 <- ifelse(length(seed1)>=2,sample(seed1,1),seed1)
		#Assign a name of the selected species to seed
		seed <- names(dist.opt)[seed1]

		com.mat <- as.matrix(com.ori.mat[seed,seed])
		colnames(com.mat) <- seed
		rownames(com.mat) <- seed
	}else{
	#if it is not the first iteration
	#Remove the row named "opt"
		com.mat <- as.matrix(com.dist)
		non.opt <- !(grepl("opt",colnames(com.mat)))
		com.mat <- com.mat[non.opt,non.opt]
	}
	#Assign names of candidate species to cand
	cand <- colnames(com.ori.mat)[!colnames(com.ori.mat) %in% colnames(com.mat)] 

	#Add "opt" to cand
	cand <- c(cand,"opt")
	#Generate a trait dissimilarity matrix of the candidates
	com.cand.mat <- com.ori.mat[cand,cand]
	#Judge whether a row is "opt" or not
	non.opt <- !(grepl("opt",colnames(com.cand.mat)))
	#List distances from an optimum trait value
	dist.opt <- com.cand.mat["opt",non.opt]
	#Assign id of the species closest ot the optimum to add1
	add1 <- which(dist.opt==min(dist.opt))
	#Sample one species randomly if tie exists
	add1 <- ifelse(length(add1)>=2,sample(add1,1),add1)
	#Assign a name of the selected species to add
	add <- names(dist.opt)[add1]

	#Assign names of species in a local community to exist
	exist <- c(add,colnames(com.mat))
	#Generate a new local community
	com.mat <- com.ori.mat[exist,exist]

	#The last row "opt" should be restored.
	opt.row <- com.ori.opt["opt",colnames(com.mat)]
	opt.col <- c(com.ori.opt[rownames(com.mat),"opt"],0)
	com.mat <- rbind(com.mat,opt.row)
	com.mat <- cbind(com.mat,opt.col)
	rownames(com.mat)[nrow(com.mat)] <- "opt"
	colnames(com.mat)[ncol(com.mat)] <- "opt"
	#Restore com.mat into class dist object
	res <- as.dist(com.mat)
	return(res)
	}

#A function to add one species based on similarity
#com.ori.dist is a class dist object of
#a dissimilarity matrix of trait(s) in a species pool
#that includes distances from an ideal trait value(s) named "opt" as the last row
#com.dist is the subset generated in the last iteration
add_via_comp=function(com.ori.dist,com.dist=NULL){
	com.ori.mat <- as.matrix(com.ori.dist)
	com.ori.opt <- com.ori.mat
	#Remove the row named "opt"
	non.opt <- !(grepl("opt",colnames(com.ori.mat)))
	com.ori.mat <- com.ori.mat[non.opt,non.opt]
	#if it is the first iteration
	if(is.null(com.dist)) {
		#Choose the first species randomly
		seed <- sample(colnames(com.ori.mat),1)
		com.mat <- as.matrix(com.ori.mat[seed,seed])
		colnames(com.mat) <- seed
		rownames(com.mat) <- seed
	}else{
	#if it is not the first iteration
	#Remove the row named "opt"
		com.mat <- as.matrix(com.dist)
		non.opt <- !(grepl("opt",colnames(com.mat)))
		com.mat <- com.mat[non.opt,non.opt]
	}

	#A sub function to return the minimum neighbor distance
	#for a candidate of recruitment
	dist_if_added <- function(sp){
		#Assign names of species in a temporal community to exist
		exist <- c(colnames(com.mat),sp)
		#Trait dissimilarity matrix for a temporal community
		com.added.mat <- com.ori.mat[exist,exist]
		#Return the minimum neighbor distance
		non.sp <- !(grepl(sp,colnames(com.added.mat)))
		dist.sp <- com.added.mat[sp,non.sp]
		return(min(dist.sp))
	}

	#Assign names of candidate species to cand
	cand <- colnames(com.ori.mat)[!colnames(com.ori.mat) %in% colnames(com.mat)] 
	#Assign the name of the species that is most dissimilar with
	#the spcies in a local community to add
	res_added <- sapply(cand,dist_if_added)
	res_added_max <- which(res_added==max(res_added))
	add <- cand[ifelse(length(res_added_max)>=2,sample(res_added_max,1),res_added_max)]
	#Assign names of species in a local community to exist
	exist <- c(add,colnames(com.mat))
	#Generate a new local community
	com.mat <- com.ori.mat[exist,exist]

	#The last row "opt" should be restored.
	opt.row <- com.ori.opt["opt",colnames(com.mat)]
	opt.col <- c(com.ori.opt[rownames(com.mat),"opt"],0)
	com.mat <- rbind(com.mat,opt.row)
	com.mat <- cbind(com.mat,opt.col)
	rownames(com.mat)[nrow(com.mat)] <- "opt"
	colnames(com.mat)[ncol(com.mat)] <- "opt"
	#Restore com.mat into class dist object
	res <- as.dist(com.mat)
	return(res)
	}

	
####################################################################
#Functions for assembly scenarios
####################################################################
#com.dist (or com.ori.dist) is a class dist object of trait dissimilarity
#for use in an assembly simulation.
#com.dist.m is a similar object for calculation of metrics
#We need the two separate arguments to test the problem of scaling.

#R-
random_until=function(n.final,com.dist,com.dist.m){
	com.mat <- as.matrix(com.dist)
	#Remove species randomly to the specified richness, n.final
	while(nrow(com.mat)-1>n.final){
		com.dist <- remove_randomly(com.dist)
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#E-
filter_until=function(n.final, com.dist,com.dist.m){
	com.mat <- as.matrix(com.dist)
	#Remove species based on environment to the specified richness, n.final
	while(nrow(com.mat)-1>n.final){
		com.dist <- remove_via_filt(com.dist)
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#E+
filter_upto=function(n.final,com.ori.dist,com.dist.m,prob){
	#Prepare a vacant local community
	com.dist <- NULL
	com.mat <- matrix()
	#Choose species based on environment up to the specified richness, n.final
	while(nrow(com.mat)<=n.final){
		com.dist <- add_via_filt(com.ori.dist,com.dist)
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#B-
compete_until=function(n.final, com.dist,com.dist.m){
	com.mat <- as.matrix(com.dist)
	#Remove species based on similarity to the specified richness, n.final
	while(nrow(com.mat)-1>n.final){
		com.dist <- remove_via_comp(com.dist)
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#B+
compete_upto=function(n.final,com.ori.dist,com.dist.m){
	#Prepare a vacant local community
	com.dist <- NULL
	com.mat <- matrix()
	#Choose species based on similarity up to the specified richness, n.final
	while(nrow(com.mat)<=n.final){
		com.dist <- add_via_comp(com.ori.dist,com.dist)
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}


#ER-
#The argument prob is a probability to 
#choose species based on environment
filrand_until=function(n.final,com.dist,com.dist.m,prob){
	com.mat <- as.matrix(com.dist)
	while(nrow(com.mat)-1>n.final){
		#Generate a random number for each iteration
		if(runif(1)<=prob){
		#Remove a species based on environment
			com.dist <- remove_via_filt(com.dist)
		}else{
		#Remove a species randomly
			com.dist <- remove_randomly(com.dist)
		}
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#BR-
comrand_until=function(n.final,com.dist,com.dist.m,prob){
	com.mat <- as.matrix(com.dist)
	while(nrow(com.mat)-1>n.final){
		#Generate a random number for each iteration
		if(runif(1)<=prob){
		#Remove a species based on similarity
			com.dist <- remove_via_comp(com.dist)
		}else{
		#Remove a species randomly
			com.dist <- remove_randomly(com.dist)
		}
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#ER+
filrand_upto=function(n.final,com.ori.dist,com.dist.m,prob){
	#Prepare a vacant local community
	com.dist <- NULL
	com.mat <- matrix()
	while(nrow(com.mat)<=n.final){
		#Generate a random number for each iteration
		if(runif(1)<=prob){
		#Choose a species based on environment
			com.dist <- add_via_filt(com.ori.dist,com.dist)
		}else{
		#Choose a species randomly
			com.dist <- add_randomly(com.ori.dist,com.dist)
		}
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#BR+
comrand_upto=function(n.final,com.ori.dist,com.dist.m,prob){
	#Prepare a vacant local community
	com.dist <- NULL
	com.mat <- matrix()
	while(nrow(com.mat)<=n.final){
		#Generate a random number for each iteration
		if(runif(1)<=prob){
		#Choose a species based on environment
			com.dist <- add_via_comp(com.ori.dist,com.dist)
		}else{
		#Choose a species randomly
			com.dist <- add_randomly(com.ori.dist,com.dist)
		}
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#EB-
filcom_until=function(n.final,com.dist,com.dist.m,prob){
	com.mat <- as.matrix(com.dist)
	while(nrow(com.mat)-1>n.final){
		#Generate a random number for each iteration
		if(runif(1)<=prob){
		#Remove a species based on environment
			com.dist <- remove_via_filt(com.dist)
		}else{
		#Remove a species based on similarity
			com.dist <- remove_via_comp(com.dist)
		}
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#EB+
filcom_upto=function(n.final,com.ori.dist,com.dist.m,prob){
	#Prepare a vacant local community
	com.dist <- NULL
	com.mat <- matrix()
	while(nrow(com.mat)<=n.final){
		#Generate a random number for each iteration
		if(runif(1)<=prob){
		#Choose a species based on environment
			com.dist <- add_via_filt(com.ori.dist,com.dist)
		}else{
		#Choose a species based on similarity
			com.dist <- add_via_comp(com.ori.dist,com.dist)
		}
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#EBR-
comfilrand_until=function(n.final,com.dist,com.dist.m){
	com.mat <- as.matrix(com.dist)
	while(nrow(com.mat)-1>n.final){
		#Generate a random number for each iteration
		rand <- runif(1)
		if(rand<=1/3){
		#Remove a species based on environment
			com.dist <- remove_via_filt(com.dist)
		}else if(rand>2/3){
		#Remove a species based on similarity
			com.dist <- remove_via_comp(com.dist)
		}else{
		#Remove a species randomly
			com.dist <- remove_randomly(com.dist)
		}
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#EBR+
comfilrand_upto=function(n.final,com.ori.dist,com.dist.m){
	#Prepare a vacant local community
	com.dist <- NULL
	com.mat <- matrix()
	while(nrow(com.mat)<=n.final){
		#Generate a random number for each iteration
		seed <- runif(1)
		if(seed<=1/3){
		#Choose a species based on environment
			com.dist <- add_via_filt(com.ori.dist,com.dist)
		}else if(seed>2/3){
		#Choose a species based on similarity
			com.dist <- add_via_comp(com.ori.dist,com.dist)
		}else{
		#Choose a species randomly
			com.dist <- add_randomly(com.ori.dist,com.dist)
		}
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}


#RREE+
rand.p.fil_upto=function(n.final,com.ori.dist,com.dist.m){
	#Prepare a vacant local community
	com.dist <- NULL
	com.mat <- matrix()
	#the former half
	while(nrow(com.mat)<=n.final/2){
	#Choose a species randomly
		com.dist <- add_randomly(com.ori.dist,com.dist)
		com.mat <- as.matrix(com.dist)
		}
	#the latter half
	while(nrow(com.mat)<=n.final){
		#Choose a species based on environment
		com.dist <- add_via_filt(com.ori.dist,com.dist)
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#EERR+
fil.p.rand_upto=function(n.final,com.ori.dist,com.dist.m){
	#Prepare a vacant local community
	com.dist <- NULL
	com.mat <- matrix()
	#the former half
	while(nrow(com.mat)<=n.final/2){
		#Choose a species based on environment
		com.dist <- add_via_filt(com.ori.dist,com.dist)
		com.mat <- as.matrix(com.dist)
		}
	while(nrow(com.mat)<=n.final){
		#Choose a species randomly
		com.dist <- add_randomly(com.ori.dist,com.dist)
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#RRBB+
rand.p.com_upto=function(n.final,com.ori.dist,com.dist.m){
	#Prepare a vacant local community
	com.dist <- NULL
	com.mat <- matrix()
	#the former half
	while(nrow(com.mat)<=n.final/2){
		#Choose a species randomly
		com.dist <- add_randomly(com.ori.dist,com.dist)
		com.mat <- as.matrix(com.dist)
		}
	while(nrow(com.mat)<=n.final){
		#Choose a species based on similarity
		com.dist <- add_via_comp(com.ori.dist,com.dist)
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

#BBRR+
com.p.rand_upto=function(n.final,com.ori.dist,com.dist.m){
	#Prepare a vacant local community
	com.dist <- NULL
	com.mat <- matrix()
	#the former half
	while(nrow(com.mat)<=n.final/2){
		#Choose a species randomly
		com.dist <- add_via_comp(com.ori.dist,com.dist)
		com.mat <- as.matrix(com.dist)
		}
	while(nrow(com.mat)<=n.final){
		#Choose a species based on similarity
		com.dist <- add_randomly(com.ori.dist,com.dist)
		com.mat <- as.matrix(com.dist)
		}
	#Remove the raw "opt"
	non.opt <- colnames(com.mat)[!(grepl("opt",colnames(com.mat)))]
	com.mat.m <- as.matrix(com.dist.m)
	com.mat <- com.mat.m[non.opt,non.opt]
	return(as.dist(com.mat))
	}

####################################################################
#Functions for simulation
####################################################################

#Function to add an optimal trait value(s) to a data frame of a trait value(s).
#An object com should be a data frame of trait values
#whose first column should be species label named "sp".
#Continuous trait values should not be log-transformed but 
#should be scaled to mean = 0 and sd = 1.
my.optim <- function(com){
	rownames(com) <- com$sp
	opt <- com[1,]
	rownames(opt) <- "opt"
	#For each of the traits
	for(i in 1:ncol(com)){
		if(grepl("^[01]*$",paste(com[,i],collapse=""))){
		#For binomial traits
			opt[i] <- sample(c(0,1),1)
		}else if(is.numeric(com[,i])){
		#For continuous traits
			opt[i] <- runif(1,min(com[,i]),max(com[,i]))
		}else{
		#For other traits
			opt[i] <- sample(com[,i],1)
		}
	}
	res <- rbind(com,opt)
	return(res[-1])
}

#Main function of the simulation
#i:A dummy argument for sfLapply()
#fun:A function for assembly simulation
#trait:Data frame of a trait values for a simulation
#trait.m:Data frame of a trait values for calculation of metrics
#snum:Local species richness
#gower:If TRUE, use Gower distance as a substitute of 
#Euclidean distance for calculation of metrics.
my.sim <- function(i,fun,trait,trait.m,snum,gower=F,...) {
	#Transform data frame of traits for calculation of metrics into dissimilarity matrix
	if(gower){
		trait.dist.m <- gowdis(trait.m[-1])
	}else{
		trait.dist.m <- dist(trait.m[-1])
	}

	#Generate optimal traits
	trait.opt <- my.optim(trait)

	#Transform data frame of traits for simulation into dissimilarity matrix
	if(gower){
		trait.dist <- gowdis(trait.opt)
	}else{
		trait.dist <- dist(trait.opt)
	}
	
	#Generate a simulated local community
	res <- try(fun(snum,trait.dist,trait.dist.m,...))
	return(res)
}

####################################################################
#Functions for single trait metrics
####################################################################
#Function for sample variance
var2 <- function(x){
	x <- na.omit(x)
	res <- (length(x)-1)/length(x)*var(x)
	return(res)
}

#Function for sample standard deviation
sd2 <- function(x){
	res <- sqrt(var2(x))
	return(res)
}


#Variance
var.s <- function (com.dist,trait){
	com.mat <- as.matrix(com.dist)
	trait <- trait[colnames(com.mat)]
	return(var2(trait))
}

#Range
range.s <- function (com.dist){
	return(max(com.dist))
}


#MNND
mnnd <- function (com.dist){
	com.mat <- as.matrix(com.dist)
	fun3 <- function(x) as.numeric(sub("^0$",NA,x))
	com.mat <- apply(com.mat,1,fun3)
	return(mean(apply(com.mat,1,min,na.rm=T)))
}

#MinNND
minnnd <- function (com.dist){
	com.mat <- as.matrix(com.dist)
	fun3 <- function(x) as.numeric(sub("^0$",NA,x))
	com.mat <- apply(com.mat,1,fun3)
	return(min(apply(com.mat,1,min,na.rm=T)))
}


#SDNN
sdnn <- function (com.dist){
	com.mat <- as.matrix(com.dist)
	#Substitute 0 with NA
	fun3 <- function(x) as.numeric(sub("^0$",NA,x))
	com.mat <- apply(com.mat,1,fun3)
	return(sd2(apply(com.mat,1,min,na.rm=T)))
}

#SDND
sdnd <- function (com.dist,trait){
	com.mat <- as.matrix(com.dist)
	trait <- trait[colnames(com.mat)]
	trait <- sort(trait)
	return(sd2(diff(trait)))
}

#SDNNr
sdnnr <- function (com.dist){
	com.mat <- as.matrix(com.dist)
	fun3 <- function(x) as.numeric(sub("^0$",NA,x))
	com.mat <- apply(com.mat,1,fun3)
	return(sd2(apply(com.mat,1,min,na.rm=T))/max(com.dist))
}

#SDNDr
sdndr <- function (com.dist,trait){
	com.mat <- as.matrix(com.dist)
	trait <- trait[colnames(com.mat)]
	trait <- sort(trait)
	return(sd2(diff(trait))/diff(range(trait)))
}

#Kurtosis
kurtosis.s <- function (com.dist,trait){
	com.mat <- as.matrix(com.dist)
	trait <- trait[colnames(com.mat)]
	return(kurtosis(trait))
}

####################################################################
#Functions for multi-traits metrics
####################################################################

#Min/Max MST
mmmst <- function (com.dist){
	com.mat <- as.matrix(com.dist)
	traits.mst <- mst(com.mat)
	traits.res <- com.mat*as.matrix(traits.mst)
	fun3 <- function(x) as.numeric(sub("^0$",NA,x))
	traits.res <- apply(traits.res,1,fun3)
	res <- min(traits.res,na.rm=T)/max(traits.res,na.rm=T)
	return(res)
}

#FRic, FEve, FDiv, FDis
fd <- function (com.dist){
	return(dbFD(com.dist,stand.x=F))
}

#FD
petchey <- function(com.dist,tr,gower){
	com.mat <- as.matrix(com.dist)
	com <- t(as.matrix(as.numeric(tr[,1] %in% colnames(com.mat))))
	colnames(com) <- tr[,1]
	if(gower){
		tree <- hclust(gowdis(tr[-1]),method="average")
	}else{
		tree <- hclust(dist(tr[-1]),method="average")
	}
	res <- treedive(com,tree)
	return(res)
}
